@@ -2,11 +2,11 @@
 
 #written by Jeremy M. Beaulieu and Jeffrey C. Oliver
 
-ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=FALSE, rate.cat, ntraits=NULL, charnum=NULL, rate.mat=NULL, model=c("ER", "SYM", "ARD"), root.p=NULL){
-	
+ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=FALSE, rate.cat, ntraits=NULL, charnum=NULL, rate.mat=NULL, model=c("ER", "SYM", "ARD"), root.p=NULL, get.likelihood=FALSE){
+
 	#Note: Does not like zero branches at the tips. Here I extend these branches by just a bit:
 	phy$edge.length[phy$edge.length<=1e-5]=1e-5
-	
+
 	if(hrm==FALSE){
 		if(ntraits==1){
 			data.sort<-data.frame(data[,charnum+1],data[,charnum+1],row.names=data[,1])
@@ -42,11 +42,11 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
             drop.states <- col.sums[which(col.sums == row.sums)]
 			rate[is.na(rate)]<-max(rate,na.rm=TRUE)+1
 		}
-		#Makes a matrix of tip states and empty cells corresponding 
-		#to ancestral nodes during the optimization process.	
+		#Makes a matrix of tip states and empty cells corresponding
+		#to ancestral nodes during the optimization process.
 		x <- data.sort[,1]
 		TIPS <- 1:nb.tip
-		
+
 		for(i in 1:nb.tip){
 			if(is.na(x[i])){x[i]=2}
 		}
@@ -120,7 +120,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 			obj <- NULL
 			nb.tip<-length(phy$tip.label)
 			nb.node <- phy$Nnode
-			
+
 			if(is.null(rate.mat)){
 				rate<-rate.mat.maker(hrm=FALSE,ntraits=ntraits,nstates=nl,model=model)
 				rate[is.na(rate)]<-max(rate,na.rm=TRUE)+1
@@ -129,13 +129,13 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 				rate<-rate.mat
 				rate[is.na(rate)]<-max(rate,na.rm=TRUE)+1
 			}
-			
+
 			stateTable <- NULL # will hold 0s and 1s for likelihoods of each state at tip
 			for(column in 1:nl){
 				stateTable <- cbind(stateTable,factored[,column])
 			}
 			colnames(stateTable) <- colnames(factored)
-			
+
 			ancestral <- matrix(0,nb.node,nl) # all likelihoods at ancestral nodes will be 0
 			liks <- rbind(stateTable,ancestral) # combine tip likelihoods & ancestral likelihoods
 			rownames(liks) <- NULL
@@ -153,7 +153,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 			}
 			x<-data.sort[,1]
 			y<-data.sort[,2]
-			
+
 			liks <- matrix(0, nb.tip + nb.node, nl^k)
 			TIPS <- 1:nb.tip
 			for(i in 1:nb.tip){
@@ -186,11 +186,11 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 			else{
 				rate<-rate.mat
 				rate[is.na(rate)]<-max(rate,na.rm=TRUE)+1
-			}			
+			}
 			x<-data.sort[,1]
 			y<-data.sort[,2]
 			z<-data.sort[,3]
-			
+
 			liks <- matrix(0, nb.tip + nb.node, nl^k)
 			TIPS <- 1:nb.tip
 			for(i in 1:nb.tip){
@@ -237,14 +237,14 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 				#If y and z is ambiguous but x is not:
 				if(x[i]==0 & y[i]==2 & z[i]==2){liks[i,c(1,3,4,7)]=1}
 				if(x[i]==1 & y[i]==2 & z[i]==2){liks[i,c(2,5,6,8)]=1}
-				#All states are ambiguous:			
+				#All states are ambiguous:
 				if(x[i]==2 & y[i]==2 & z[i]==2){liks[i,1:8]=1}
 			}
 		}
 		Q <- matrix(0, nl^k, nl^k)
 		tranQ <- matrix(0, nl^k, nl^k)
 	}
-	
+
     if(length(drop.states > 0)){
         liks[,drop.states] <- 0
     }
@@ -255,123 +255,191 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 	phy <- reorder(phy, "pruningwise")
 	TIPS <- 1:nb.tip
 	anc <- unique(phy$edge[,1])
-	if(method=="joint"){
+
+    if(method=="joint"){
+        if(!is.null(phy$node.label)){
+            tip.state.vector <- rep(NA, Ntip(phy))
+            #We remove the first, because the root state probability comes in through root.p:
+            known.state.vector <- phy$node.label
+            known.state.vector <- c(tip.state.vector, known.state.vector)
+        }else{
+            tip.state.vector <- rep(NA, Ntip(phy))
+            known.state.vector <- rep(NA, Nnode(phy))
+            known.state.vector <- c(tip.state.vector, known.state.vector)
+        }
 		lik.states<-numeric(nb.tip + nb.node)
-		comp<-matrix(0,nb.tip + nb.node,ncol(liks))
-		for (i  in seq(from = 1, length.out = nb.node)) {
-			#The ancestral node at row i is called focal:
-			focal <- anc[i]
-			#Get descendant information of focal:
-			desRows<-which(phy$edge[,1]==focal)
-			#Get node information for each descendant:
-			desNodes<-phy$edge[desRows,2]
-			#Initiates a loop to check if any nodes are tips:
-			for (desIndex in sequence(length(desRows))){
-				#If a tip calculate C_y(i) for the tips and stores in liks matrix:
-				if(any(desNodes[desIndex]==phy$edge[,1])==FALSE){
-					if(hrm==TRUE){
-						v <- c(rep(1, k*rate.cat))
-					}
-					if(hrm==FALSE){
-						v <- c(rep(1, nl^k))
-					}
-					Pij <- expm(Q * phy$edge.length[desRows[desIndex]], method=c("Ward77"))
-					v = v * liks[desNodes[desIndex],]
-					for(i in 1:dim(Pij)[1]){
-						L <- Pij[i,] * v
-						liks[desNodes[desIndex],i] <- max(L)
-						comp[desNodes[desIndex],i] <- which.max(L==max(L))[1]
-					}					
-				}
-			}
-			#Collects t_z, or the branch subtending focal:
-			tz<-phy$edge.length[which(phy$edge[,2] == focal)]	
-			if(length(tz)==0){
-				#The focal node is the root, calculate P_k:
-				root.state=1
-				for (desIndex in sequence(length(desRows))){
-					#This is the basic marginal calculation:
-					root.state <- root.state * liks[desNodes[desIndex],]
-				}
-                equil.root <- NULL
-                for(i in 1:ncol(Q)){
-                    posrows <- which(Q[,i] >= 0)
-                    rowsum <- sum(Q[posrows,i])
-                    poscols <- which(Q[i,] >= 0)
-                    colsum <- sum(Q[i,poscols])
-                    equil.root <- c(equil.root,rowsum/(rowsum+colsum))
+		pupko.L <- matrix(NA,nrow=nb.tip + nb.node,ncol(liks))
+		pupko.C <- matrix(NA,nrow=nb.tip + nb.node,ncol(liks))
+        for (i  in seq(from = 1, length.out = nb.node)) {
+            #The ancestral node at row i is called focal:
+            focal <- anc[i]
+            #Get descendant information of focal:
+            desRows<-which(phy$edge[,1]==focal)
+            #Get node information for each descendant:
+            desNodes<-phy$edge[desRows,2]
+            #Initiates a loop to check if any nodes are tips:
+            for (desIndex in sequence(length(desRows))){
+                #If a tip calculate C_y(i) for the tips and stores in liks matrix:
+                if(any(desNodes[desIndex]==phy$edge[,1])==FALSE){
+                    if(hrm==TRUE){
+                        v <- c(rep(1, k*rate.cat))
+                    }
+                    if(hrm==FALSE){
+                        v <- c(rep(1, nl^k))
+                    }
+                    Pij <- expm(Q * phy$edge.length[desRows[desIndex]], method=c("Ward77"))
+                    #Pij <- matrix(c(0.7, 0.45, 0.3, 0.55), 2, 2)
+                    v <- v * liks[desNodes[desIndex],]
+                    L <- Pij %*% v
+                    #liks: rows are taxa + internal nodes, cols are # states
+                    if(is.na(known.state.vector[focal])){
+                        pupko.L[desNodes[desIndex],] <- L
+                        pupko.C[desNodes[desIndex],] <- which.is.max(L==max(L))
+                    }else{
+                        pupko.L[desNodes[desIndex],] <- L[known.state.vector[focal],]
+                        pupko.C[desNodes[desIndex],] <- known.state.vector[focal]
+                    }
                 }
-                if (is.null(root.p)){
-                    flat.root = equil.root
-                    k.rates <- 1/length(which(!is.na(equil.root)))
-                    flat.root[!is.na(flat.root)] = k.rates
-                    flat.root[is.na(flat.root)] = 0
-                    liks[focal, ] <- root.state
+            }
+            #Collects t_z, or the branch subtending focal:
+            tz <- phy$edge.length[which(phy$edge[,2] == focal)]
+            if(length(tz)==0){
+                #The focal node is the root, calculate P_k:
+                root.state=1
+                for (desIndex in sequence(length(desRows))){
+                    #This is the basic marginal calculation:
+                    root.state <- root.state * pupko.L[desNodes[desIndex],]
                 }
-                else{
-                    if(is.character(root.p)){
-                        # root.p==yang will fix root probabilities based on the inferred rates: q10/(q01+q10), q01/(q01+q10), etc.
-                        if(root.p == "yang"){
-                            Q.tmp <- Q
-                            diag(Q.tmp) = 0
-                            root.p = colSums(Q.tmp) / sum(Q.tmp)
-                            liks[focal, ] <- root.state * root.p
+                if(is.na(known.state.vector[focal])){
+                    equil.root <- NULL
+                    for(i in 1:ncol(Q)){
+                        posrows <- which(Q[,i] >= 0)
+                        rowsum <- sum(Q[posrows,i])
+                        poscols <- which(Q[i,] >= 0)
+                        colsum <- sum(Q[i,poscols])
+                        equil.root <- c(equil.root,rowsum/(rowsum+colsum))
+                    }
+                    if (is.null(root.p)){
+                        if(is.na(known.state.vector[focal])){
+                            flat.root = equil.root
+                            k.rates <- 1/length(which(!is.na(equil.root)))
+                            flat.root[!is.na(flat.root)] = k.rates
+                            flat.root[is.na(flat.root)] = 0
+                            #root.p <- c(.6,.4)
+                            root.p <- flat.root
+                            pupko.L[focal, ] <- root.state
                         }else{
-                            # root.p==maddfitz will fix root probabilities according to FitzJohn et al 2009 Eq. 10:
-                            root.p = root.state / sum(root.state)
-                            liks[focal,] <- root.state * root.p
+                            root.p <- rep(0, dim(Q)[2])
+                            root.p[known.state.vector[focal]] <- 1
+                            pupko.L[focal, ] <- root.state
                         }
                     }
-                    # root.p!==NULL will fix root probabilities based on user supplied vector:
                     else{
-                        liks[focal, ] <- root.state * root.p
+                        if(is.character(root.p)){
+                            # root.p==yang will fix root probabilities based on the inferred rates: q10/(q01+q10), q01/(q01+q10), etc.
+                            if(root.p == "yang"){
+                                Q.tmp <- Q
+                                diag(Q.tmp) = 0
+                                root.p = colSums(Q.tmp) / sum(Q.tmp)
+                                pupko.L[focal, ] <- root.state
+                            }else{
+                                # root.p==maddfitz will fix root probabilities according to FitzJohn et al 2009 Eq. 10:
+                                root.p <- root.state / sum(root.state)
+                                pupko.L[focal,] <- root.state
+                            }
+                        }
+                        # root.p!==NULL will fix root probabilities based on user supplied vector:
+                        else{
+                            root.p <- root.p
+                            pupko.L[focal, ] <- root.state
+                        }
+                    }
+                }else{
+                    root.p = rep(0, dim(Q)[1])
+                    root.p[known.state.vector[focal]] <- 1
+                    pupko.L[focal, ] <- root.state
+                }
+            }
+            #All other internal nodes, except the root:
+            else{
+                #Calculates P_ij(t_z):
+                Pij <- expm(Q * tz, method=c("Ward77"))
+                #Pij <- matrix(c(0.7, 0.45, 0.3, 0.55), 2, 2)
+                #Calculates L_z(i):
+                if(hrm==TRUE){
+                    v <- c(rep(1, k*rate.cat))
+                }
+                if(hrm==FALSE){
+                    v <- c(rep(1, nl^k))
+                }
+                if(is.na(known.state.vector[focal])){
+                    for (desIndex in sequence(length(desRows))){
+                        v <- v * pupko.L[desNodes[desIndex],]
+                    }
+                    focalRow <- which(phy$edge[,2]==focal)
+                    motherRow <- which(phy$edge[,1]==phy$edge[focalRow,1])
+                    motherNode <- phy$edge[focalRow,1]
+                    if(is.na(known.state.vector[motherNode])){
+                        for(row.index in 1:dim(Pij)[1]){
+                            L <- Pij[row.index,] * v
+                            pupko.L[focal, row.index] <- max(L)
+                            pupko.C[focal, row.index] <- which.is.max(L)
+                        }
+                    }else{
+                        L <- Pij[known.state.vector[motherNode],] * v
+                        pupko.L[focal,] <- L
+                        pupko.C[focal,] <- which.is.max(L)
+                    }
+                }else{
+                    for (desIndex in sequence(length(desRows))){
+                        v <- v * pupko.L[desNodes[desIndex],]
+                    }
+                    focalRow <- which(phy$edge[,2] == focal)
+                    motherRow <- which(phy$edge[,1] == phy$edge[focalRow,1])
+                    motherNode <- phy$edge[focalRow,1]
+                    if(is.na(known.state.vector[motherNode])){
+                        for(row.index in 1:dim(Pij)[1]){
+                            L <- Pij[row.index,] * v
+                            pupko.L[focal, row.index] <- L[known.state.vector[focal]]
+                            pupko.C[focal, row.index] <- known.state.vector[focal]
+                        }
+                    }else{
+                        L <- Pij[known.state.vector[motherNode],] * v
+                        pupko.L[focal,] <- L[known.state.vector[focal]]
+                        pupko.C[focal,] <- known.state.vector[focal]
                     }
                 }
-
-			}
-			#All other internal nodes, except the root:
-			else{
-				#Calculates P_ij(t_z):
-				Pij <- expm(Q * tz, method=c("Ward77"))
-				#Calculates L_z(i):
-				if(hrm==TRUE){
-					v<-c(rep(1, k*rate.cat))
-				}
-				if(hrm==FALSE){
-					v<-c(rep(1, nl^k))
-				}
-				for (desIndex in sequence(length(desRows))){
-					v = v * liks[desNodes[desIndex],]
-				}
-				for(i in 1:dim(Pij)[1]){
-					L <- Pij[i,] * v
-					liks[focal,i] <- max(L)
-					comp[focal,i] <- which.max(L==max(L))[1]
-				}
-				sum.tot <- sum(liks[focal,])
-				liks[focal,]=liks[focal,]/sum.tot
-				
-#				if(sum(liks[focal,])<1e-200){
-					#Kicks in arbitrary precision calculations: 
-#					library(Rmpfr)
-#					liks <- mpfr(liks, 15)	
-#				}
-			}
-		}
-		root <- nb.tip + 1L
-        lik.states[root] <- which(liks[root,]==max(liks[root,]))
-		N <- dim(phy$edge)[1]
-		for(i in N:1){
-			anc <- phy$edge[i,1]
-			des <- phy$edge[i,2]
-			lik.states[des] <- comp[des,lik.states[anc]]
-		}
-		#For later use:
-		#logl <- as.numeric(log(liks[root,lik.states[root]]))
-		#Outputs likeliest tip states
-		obj$lik.tip.states <- lik.states[TIPS]
-		#Outputs likeliest node states
-		obj$lik.anc.states <- lik.states[-TIPS]
+                if(sum(pupko.L[focal,])<1e-200){
+                    cat("Kicking in arbitrary precision package Rmpfr due to very low probabilities.\n")
+                    #Kicks in arbitrary precision calculations:
+                    pupko.L <- mpfr(pupko.L, 15)
+                }
+            }
+        }
+        root <- nb.tip + 1L
+        if(get.likelihood == TRUE){
+            loglik <- log(sum(exp(log(root.p)+log(pupko.L[root,]))))
+            return(as.numeric(loglik))
+        }else{
+            root <- nb.tip + 1L
+            if(is.na(known.state.vector[root])){
+                pupko.L[root,] <- log(root.p)+log(pupko.L[root,])
+                lik.states[root] <- which(pupko.L[root,] == max(pupko.L[root,]))[1]
+            }else{
+                lik.states[root] <- known.state.vector[root]
+            }
+            N <- dim(phy$edge)[1]
+            for(i in N:1){
+                anc <- phy$edge[i,1]
+                des <- phy$edge[i,2]
+                lik.states[des] <- pupko.C[des,lik.states[anc]]
+            }
+            #Outputs likeliest tip states
+            obj$lik.tip.states <- lik.states[TIPS]
+            #Outputs likeliest node states
+            obj$lik.anc.states <- lik.states[-TIPS]
+            return(obj)
+        }
 	}
 	if(method=="marginal"){
 		#A temporary likelihood matrix so that the original does not get written over:
@@ -380,7 +448,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 		tranQ<-t(Q)
 		comp<-matrix(0,nb.tip + nb.node,ncol(liks))
 		#The first down-pass: The same algorithm as in the main function to calculate the conditional likelihood at each node:
-		for (i  in seq(from = 1, length.out = nb.node)) {
+		for (i in seq(from = 1, length.out = nb.node)) {
 			#the ancestral node at row i is called focal
 			focal <- anc[i]
 			#Get descendant information of focal
@@ -454,12 +522,12 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 				if(motherNode!=root){
 					v <- expm(tranQ * phy$edge.length[which(phy$edge[,2]==motherNode)], method=c("Ward77")) %*% liks.up[motherNode,]
 				}
-				#If the mother is the root then just use the marginal. This can also be the prior, which I think is the equilibrium frequency. 
+				#If the mother is the root then just use the marginal. This can also be the prior, which I think is the equilibrium frequency.
 				#But for now we are just going to use the marginal at the root -- it is unclear what Mesquite does.
 				else{
 					v <- root.p
 				}
-				#Now calculate the probability that each sister is in either state. Sister can be more than 1 when the node is a polytomy. 
+				#Now calculate the probability that each sister is in either state. Sister can be more than 1 when the node is a polytomy.
 				#This is essentially calculating the product of the mothers probability and the sisters probability:
 				for (sisterIndex in sequence(length(sisterRows))){
 					v <- v*expm(Q * phy$edge.length[sisterRows[sisterIndex]], method=c("Ward77")) %*% liks.down[sisterNodes[sisterIndex],]
@@ -472,39 +540,39 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 		liks.final<-liks
 		comp <- numeric(nb.tip + nb.node)
 		#In this final pass, root is never encountered. But its OK, because root likelihoods are set after the loop:
-		for (i in seq(from = 1, length.out = nb.node-1)) { 
+		for (i in seq(from = 1, length.out = nb.node-1)) {
 			#the ancestral node at row i is called focal
 			focal <- anc[i]
 			focalRows<-which(phy$edge[,2]==focal)
-			#Now you are assessing the change along the branch subtending the focal by multiplying the probability of 
+			#Now you are assessing the change along the branch subtending the focal by multiplying the probability of
 			#everything at and above focal by the probability of the mother and all the sisters given time t:
 			v <- liks.down[focal,]*expm(tranQ * phy$edge.length[focalRows], method=c("Ward77")) %*% liks.up[focal,]
 			comp[focal] <- sum(v)
 			liks.final[focal, ] <- v/comp[focal]
 		}
-		#Now get the states for the tips (will do, not available for general use):
-		for (i in seq(from = 1, length.out = length(TIPS))) { 
-			#the ancestral node at row i is called focal
-			focal <- TIPS[i]
-			focalRows <- which(phy$edge[,2]==focal)
-			#Now you are assessing the change along the branch subtending the focal by multiplying the probability of 
-			#everything at and above focal by the probability of the mother and all the sisters given time t:
-			v <- liks.down[focal,]*expm(tranQ * phy$edge.length[focalRows], method=c("Ward77")) %*% liks.up[focal,]
-			comp[focal] <- sum(v)
-			liks.final[focal, ] <- v/comp[focal]
-		}		
-		#Just add in the marginal at the root calculated on the original downpass or if supplied by the user:
+
+        #Now get the states for the tips (will do, not available for general use):
+        liks.final[TIPS,] <- GetTipStateBruteForce(p=p, phy=phy, data=data.sort, rate.mat=rate.mat, rate.cat=rate.cat, charnum=charnum, ntraits=ntraits, model=model, root.p=root.p)
+
+        #Just add in the marginal at the root calculated on the original downpass or if supplied by the user:
 		liks.final[root,] <- liks.down[root,] * root.p
 		root.final <- liks.down[root,] * root.p
 		comproot <- sum(root.final)
 		liks.final[root,] <- root.final/comproot
-		#Reports the probabilities for all internal nodes as well as tips:
-		#Outputs likeliest tip states
-		obj$lik.tip.states <- liks.final[TIPS,]
-		#Outputs likeliest node states
-		obj$lik.anc.states <- liks.final[-TIPS,]
-	}	
-	
+
+        if(get.likelihood == TRUE){
+############NEED TO FIGURE OUT LOG COMPENSATION ISSUE --- see line 397.
+            loglik <- as.numeric(log(liks[root,lik.states[root]]))
+            return(loglik)
+        }else{
+            #Outputs likeliest tip states
+            obj$lik.tip.states <- liks.final[TIPS,]
+            #Outputs likeliest node states
+            obj$lik.anc.states <- liks.final[-TIPS,]
+            return(obj)
+        }
+	}
+
 	if(method=="scaled"){
 		comp<-matrix(0,nb.tip + nb.node,ncol(liks))
         root <- nb.tip + 1L
@@ -563,7 +631,63 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 		obj$lik.tip.states <- liks[TIPS,]
 		#Outputs likeliest node states
 		obj$lik.anc.states <- liks[-TIPS,]
-	}	
-	obj
+        return(obj)
+	}
 }
 
+
+
+#New brute force algorithm for estimating tip states.
+GetTipStateBruteForce <- function(p, phy, data, rate.mat, rate.cat, charnum, ntraits, model, root.p){
+
+    nb.tip <- length(phy$tip.label)
+    nb.node <- phy$Nnode
+
+    if(is.null(rate.cat)){
+        if(ntraits<2){
+            data.for.likelihood.function <- rate.cat.set.rayDISC(phy=phy, data=data, model=model, charnum=charnum)
+        }else{
+            data.for.likelihood.function <- rate.mat.set(phy=phy, data.sort=data, ntraits=ntraits, model=model)
+        }
+    }else{
+        data.for.likelihood.function <- rate.cat.set.corHMM(phy=phy, data.sort=data, rate.cat=rate.cat)
+    }
+
+    if(!is.null(rate.mat)){
+        rate <- rate.mat
+        data.for.likelihood.function$np <- max(rate, na.rm=TRUE)
+        rate[is.na(rate)]=max(rate, na.rm=TRUE)+1
+        data.for.likelihood.function$rate <- rate
+        data.for.likelihood.function$index.matrix <- rate.mat
+        ## for precursor type models ##
+        col.sums <- which(colSums(rate.mat, na.rm=TRUE) == 0)
+        row.sums <- which(rowSums(rate.mat, na.rm=TRUE) == 0)
+        drop.states <- col.sums[which(col.sums == row.sums)]
+
+        if(length(drop.states > 0)){
+            data.for.likelihood.function$liks[,drop.states] <- 0
+        }
+        ###############################
+    }
+
+    nodes <- unique(phy$edge[,1])
+    marginal.probs <- matrix(0, nb.tip, dim(data.for.likelihood.function$Q)[2])
+    for(taxon.index in 1:Ntip(phy)){
+        marginal.probs.tmp <- numeric(dim(data.for.likelihood.function$Q)[2])
+        nstates = which(!data.for.likelihood.function$liks[taxon.index,] == 0)
+        states.keep = data.for.likelihood.function$liks[taxon.index,]
+        for(state.index in 1:dim(data.for.likelihood.function$Q)[2]){
+            data.for.likelihood.function$liks[taxon.index,] = 0
+            data.for.likelihood.function$liks[taxon.index,state.index] = 1
+            marginal.probs.tmp[state.index] <- -dev.corhmm(p=log(p), phy=phy, liks=data.for.likelihood.function$liks, Q=data.for.likelihood.function$Q, rate=data.for.likelihood.function$rate, root.p=root.p)
+        }
+        data.for.likelihood.function$liks[taxon.index,] = states.keep
+        best.probs = max(marginal.probs.tmp[nstates])
+        marginal.probs.rescaled = marginal.probs.tmp[nstates] - best.probs
+        marginal.probs[taxon.index,nstates] = exp(marginal.probs.rescaled) / sum(exp(marginal.probs.rescaled))
+    }
+
+    tip.states <- marginal.probs[1:nb.tip,]
+    rownames(tip.states) <- phy$tip.label
+    return(tip.states)
+}
